"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
// 同步 Taro 项目中 packages/taro-webpack5-runner/src/postcss/postcss.h5.ts 中的逻辑
const path_1 = __importDefault(require("path"));
const lodash_1 = require("lodash");
const resolve_1 = require("resolve");
const platform = 'h5';
const defaultAutoprefixerOption = {
    enable: true,
    config: {
        flexbox: 'no-2009'
    }
};
const defaultPxtransformOption = {
    enable: true,
    config: {
        platform
    }
};
const defaultConstparseOption = {
    constants: [
        {
            key: 'taro-tabbar-height',
            val: '50PX'
        }
    ],
    platform
};
const defaultHtmltransformOption = {
    enable: true,
    config: {
        platform,
        removeCursorStyle: false
    }
};
const defaultUrlOption = {
    enable: true,
    config: {
        url: 'rebase'
    }
};
function isEnable(option) {
    return !option || !('enable' in option) || option.enable;
}
function isNpmPkg(name) {
    return !/^(\.|\/)/.test(name);
}
const getPostCssConfig = getConfig => {
    // 处理 Taro 编译配置，同步 taro-service/src/Config.ts 中的逻辑
    const config = getConfig(lodash_1.merge);
    const { designWidth = 750, deviceRatio, alias, h5 } = config;
    // 处理 postcss 配置，同步 taro-webpack5-runner/src/postcss/postcss.h5.ts 中的逻辑
    const _a = h5.postcss, { autoprefixer, pxtransform, htmltransform, url, cssModules } = _a, rest = __rest(_a, ["autoprefixer", "pxtransform", "htmltransform", "url", "cssModules"]);
    const plugins = [];
    plugins.push(require.resolve('postcss-import'));
    const autoprefixerOption = (0, lodash_1.merge)({}, defaultAutoprefixerOption, autoprefixer);
    if (isEnable(autoprefixerOption)) {
        plugins.push([require.resolve('autoprefixer'), autoprefixerOption]);
    }
    if (designWidth) {
        defaultPxtransformOption.config.designWidth = designWidth;
    }
    if (deviceRatio) {
        defaultPxtransformOption.config.deviceRatio = deviceRatio;
    }
    const pxtransformOption = (0, lodash_1.merge)({}, defaultPxtransformOption, pxtransform);
    if (isEnable(pxtransformOption)) {
        plugins.push([require.resolve('postcss-pxtransform'), pxtransformOption.config || {}]);
    }
    const htmlTransformOption = (0, lodash_1.merge)({}, defaultHtmltransformOption, htmltransform);
    if (isEnable(htmlTransformOption)) {
        plugins.push([require.resolve('postcss-html-transform'), htmlTransformOption.config || {}]);
    }
    plugins.push([require.resolve('postcss-plugin-constparse'), defaultConstparseOption.config || {}]);
    const aliasOption = { config: { alias } };
    plugins.push([require.resolve('./postcss-alias'), aliasOption]);
    const urlOption = (0, lodash_1.merge)({}, defaultUrlOption, url);
    if (isEnable(urlOption)) {
        plugins.push([require.resolve('postcss-url'), urlOption.config || {}]);
    }
    const appPath = process.cwd();
    for (const [name, pluginOption] of Object.entries(rest)) {
        let pluginName = name;
        if (!isEnable(pluginOption)) {
            break;
        }
        if (!isNpmPkg(pluginName)) {
            // local plugin
            pluginName = path_1.default.join(appPath, pluginName);
        }
        try {
            const pluginPath = (0, resolve_1.sync)(pluginName, { basedir: appPath });
            plugins.push([pluginPath, pluginOption.config || {}]);
        }
        catch (e) {
            const msg = e.code === 'MODULE_NOT_FOUND' ? `缺少 postcss 插件 "${pluginName}", 已忽略` : e;
            console.log(msg);
        }
    }
    return {
        plugins
    };
};
module.exports = getPostCssConfig;
