"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const JS_EXT = ['.js', '.jsx'];
const TS_EXT = ['.ts', '.tsx'];
const SCRIPT_EXT = JS_EXT.concat(TS_EXT);
function resolveMainFilePath(p, extArrs = SCRIPT_EXT) {
    const realPath = p;
    const taroEnv = 'h5';
    for (let i = 0; i < extArrs.length; i++) {
        const item = extArrs[i];
        if (taroEnv) {
            if (fs.existsSync(`${p}.${taroEnv}${item}`)) {
                return `${p}.${taroEnv}${item}`;
            }
            if (fs.existsSync(`${p}${path.sep}index.${taroEnv}${item}`)) {
                return `${p}${path.sep}index.${taroEnv}${item}`;
            }
            if (fs.existsSync(`${p.replace(/\/index$/, `.${taroEnv}/index`)}${item}`)) {
                return `${p.replace(/\/index$/, `.${taroEnv}/index`)}${item}`;
            }
        }
        if (fs.existsSync(`${p}${item}`)) {
            return `${p}${item}`;
        }
        if (fs.existsSync(`${p}${path.sep}index${item}`)) {
            return `${p}${path.sep}index${item}`;
        }
    }
    return realPath;
}
function resolveScriptPath(p) {
    return resolveMainFilePath(p);
}
function normalizePath(path) {
    return path.replace(/\\/g, '/').replace(/\/{2,}/g, '/');
}
function promoteRelativePath(fPath) {
    const fPathArr = fPath.split(path.sep);
    let dotCount = 0;
    fPathArr.forEach(item => {
        if (item.indexOf('..') >= 0) {
            dotCount++;
        }
    });
    if (dotCount === 1) {
        fPathArr.splice(0, 1, '.');
        return fPathArr.join('/');
    }
    if (dotCount > 1) {
        fPathArr.splice(0, 1);
        return fPathArr.join('/');
    }
    return normalizePath(fPath);
}
function isAliasPath(name, pathAlias = {}) {
    const prefixes = Object.keys(pathAlias);
    if (prefixes.length === 0) {
        return false;
    }
    return prefixes.includes(name) || (new RegExp(`^(${prefixes.join('|')})/`).test(name));
}
function replaceAliasPath(filePath, name, pathAlias = {}) {
    // 后续的 path.join 在遇到符号链接时将会解析为真实路径，如果
    // 这里的 filePath 没有做同样的处理，可能会导致 import 指向
    // 源代码文件，导致文件被意外修改
    filePath = fs.realpathSync(filePath);
    const prefixes = Object.keys(pathAlias);
    if (prefixes.includes(name)) {
        return promoteRelativePath(path.relative(filePath, fs.realpathSync(resolveScriptPath(pathAlias[name]))));
    }
    const reg = new RegExp(`^(${prefixes.join('|')})/(.*)`);
    name = name.replace(reg, function (_m, $1, $2) {
        return promoteRelativePath(path.relative(filePath, path.join(pathAlias[$1], $2)));
    });
    return name;
}
const pattern = /(url\(\s*['"]?)([^"')]+)(["']?\s*\))/g;
const postcssAlias = (options = {}) => {
    return {
        postcssPlugin: 'postcss-alias',
        Once(styles, { result }) {
            if (!options.alias || !Object.keys(options.alias).length)
                return;
            const opts = result.opts;
            const from = opts.from;
            styles.walkDecls(decl => {
                if (pattern.test(decl.value)) {
                    decl.value = decl.value.replace(pattern, (matched, before, url, after) => {
                        url = url.replace(/^~/, '');
                        if (isAliasPath(url, options.alias)) {
                            const newUrl = replaceAliasPath(from, url, options.alias);
                            return `${before}${newUrl}${after}`;
                        }
                        else {
                            return matched;
                        }
                    });
                }
            });
        }
    };
};
postcssAlias.postcss = true;
module.exports = postcssAlias;
